<?php

/**
 * Plugin Name: WCML Exchange Rate Auto Update
 * Description: Custom Base currency exchange rates auto updater for WCML (WooCommerce MultiLingual)
 * Plugin URI: https://www.fjobeir.com/
 * Version: 0.1.0
 * Author: Feras Jobeir
 * Author URI: https://www.fjobeir.com
 * Text Domain: wcml-exchange-rate-auto-update
 */

// Prevent script direct access
if (!defined('ABSPATH')) {
    exit;
}

class WCMLExchangeUpdater
{
    // declare two empty arrays to store errors and warnings inside
    public $errors, $warnings, $settings = array();
    // declare two empty strings to store the default currency and api key
    public $default_currency = '';
    public $_wcml_settings;

    public function __construct()
    {
        // Get the default currency symbol
        $this->default_currency = get_option('woocommerce_currency');
        // Get the currenct WCML settings
        $this->_wcml_settings = get_option('_wcml_settings');

        if (!is_string($this->default_currency) || strlen($this->default_currency) != 3) {
            // If the default currency is not properly set, exit with an error
            $this->errors[] = __('Invalid default currency', 'wcml-exchange-rate-auto-update');
            $this->wcmlru_set_messages();
        } elseif ($this->_wcml_settings === false) {
            // No WCML settings found
            $this->errors[] = __('No WCML Settings found', 'wcml-exchange-rate-auto-update');
            $this->wcmlru_set_messages();
        } else {
            // Let us check the rate updating schedule first
            switch ($this->_wcml_settings['multi_currency']['exchange_rates']['schedule']) {
                case 'daily':
                    // Check if we have already updated today
                    if ($this->weHaveAlreadyUpdatedToday()) {
                        return;
                    }
                    break;
                case 'weekly':
                    // Compare today's code with the stored day code
                    if (date('w') != $this->_wcml_settings['multi_currency']['exchange_rates']['week_day'] || $this->weHaveAlreadyUpdatedToday()) {
                        return;
                    }
                    break;
                case 'monthly':
                    // Compare today's code with the stored day code
                    if (date('j') != $this->_wcml_settings['multi_currency']['exchange_rates']['month_day'] || $this->weHaveAlreadyUpdatedToday()) {
                        return;
                    }
                    break;
            }
            // If we reached this point, let us update
            $this->wcmlru_update_exchangerate();
        }
    }
    public function weHaveAlreadyUpdatedToday()
    {
        return (date('Y-m-d') == date('Y-m-d', $this->_wcml_settings['multi_currency']['exchange_rates']['last_updated']));
    }

    public function wcmlru_update_exchangerate()
    {
        // check if fixer.io api key is provided
        $fixerio_api = get_option('wcml_exchange_rate_service_fixerio');
        if ($fixerio_api) {
            // check if the API key is provided in 'api-key'
            if (array_key_exists('api-key', $fixerio_api) && strlen($fixerio_api['api-key'])) {
                // use fixer.io to get rates
                $this->settings['request_url'] = 'http://data.fixer.io/api/latest?format=1&access_key=' . $fixerio_api['api-key'];
                $this->settings['response_rates_key'] = 'rates';
                $this->settings['source'] = 'fixer.io';
                // Do the update
                $this->wcmlru_updateNewRates();
                return;
            }
        }
        // If you reached this point, you are going to check the currency layer key
        $currencylayer_api = get_option('wcml_exchange_rate_service_currencylayer');
        if ($currencylayer_api) {
            if (array_key_exists('api-key', $currencylayer_api) && strlen($currencylayer_api['api-key'])) {
                // use currency layer to get rates
                $this->settings['request_url'] = 'http://apilayer.net/api/live?access_key=' . $currencylayer_api['api-key'];
                $this->settings['response_rates_key'] = 'quotes';
                $this->settings['source'] = 'currencylayer';
                // The monthly limit of currency layer for the free plan has been decreased to 250 only, so we will update once every 3 hours
                if (time() - $this->_wcml_settings['multi_currency']['exchange_rates']['last_updated'] < 10800) {
                    // Return if the difference between now and the last update is less then 10800 seconds (3 hours)
                    return;
                }
                // Do the update
                $this->wcmlru_updateNewRates();
                return;
            }
        }
        // If you reached this point, this means that you have not setup fixer.io nor currency layer
        $this->errors[] = __('You have not setup fixer.io nor currencylayer key', 'wcml-exchange-rate-auto-update');
        $this->wcmlru_set_messages();
    }
    public function wcmlru_returnCurrencyKey($currency)
    {
        return $this->settings['source'] == 'currencylayer' ? 'USD' . $currency : $currency;
    }
    public function wcmlru_updateNewRates()
    {
        $response = wp_remote_get($this->settings['request_url']);
        try {
            $parsed_response = \json_decode($response['body'], true);
            if ($parsed_response['success'] == true) {
                $prices = $parsed_response[$this->settings['response_rates_key']];
                if (array_key_exists($this->wcmlru_returnCurrencyKey($this->default_currency), $prices)) {
                    // Default currency to source base exchange rate
                    $default_to_base = 1 / $prices[$this->wcmlru_returnCurrencyKey($this->default_currency)];
                    // Extract the lifting charge
                    $lifting_charge = ((float)($this->_wcml_settings['multi_currency']['exchange_rates']['lifting_charge'])) / 100;
                    // Loop through our currencies array
                    foreach ($this->_wcml_settings['currency_options'] as $symbol => $value) {
                        // Check if the exists currency is also existing in currencylayer response
                        if (array_key_exists($this->wcmlru_returnCurrencyKey($symbol), $prices)) {
                            // Calculate any other currency rate by applying the following equation
                            // (Default/Currency) = (Base/Currency) * (Default/Base)
                            // Will then add the lifting charge
                            $rate = ($prices[$this->wcmlru_returnCurrencyKey($symbol)] * $default_to_base);
                            $this->_wcml_settings['currency_options'][$symbol]['rate'] = $rate + ($rate * $lifting_charge);
                        } else {
                            // Show a warning message to the user
                            $this->warnings[] = sprintf(__('%1$s rate was not provided by %2$s', 'wcml-exchange-rate-auto-update'), $key, ucfirst($this->settings['source']));
                        }
                    }
                    $this->_wcml_settings['multi_currency']['exchange_rates']['last_updated'] = time();
                    update_option('_wcml_settings', $this->_wcml_settings, 'yes');
                } else {
                    $this->errors[] = sprintf(__('Your default currency (%1$s) is not supported by %2$s', 'wcml-exchange-rate-auto-update'), $parsed_response['error']['info'], ucfirst($this->settings['source']));
                }
            } else {
                $this->errors[] = sprintf(__('Currency Layer returned the following message: %s', 'wcml-exchange-rate-auto-update'), $parsed_response['error']['info']);
            }
        } catch (Exception $e) {
            $this->errors[] = sprintf(__('Error while getting data: %s', 'wcml-exchange-rate-auto-update'), $e->getMessage());
        } finally {
            $this->wcmlru_set_messages();
        }
    }
    
    public function wcmlru_set_messages()
    {
        update_option('wcmlru_warning_messages', $this->warnings);
        update_option('wcmlru_error_messages', $this->errors);
    }
}

// Admin options & notices
require 'wcmlru_admin.php';

// Register WP Cronjob on plugin activation
register_activation_hook(__FILE__, 'wcmlru_update_schedule');

function wcmlru_update_schedule()
{
    // Get the next event timestamp
    // This is to avoid conflict if the user deactivated then reactivated the plugin again
    $update_exchangerate_timestamp = wp_next_scheduled('wcmlru_schedule');
    // check if that schedule is not already set
    if ($update_exchangerate_timestamp == false) {
        // Set an hourly schedule
        // This will run 720 times per 30 days.
        wp_schedule_event(time(), 'hourly', 'wcmlru_run_action');
    }
}

add_action('wcmlru_run_action', 'wcmlru_run_rate_updater');
function wcmlru_run_rate_updater()
{
    $updater = new WCMLExchangeUpdater();
}